// Main - Main functions
// Copyright (C) 2002-2006 Jerome Martinez, Zen@MediaArea.net
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#include "Common/Core.h"
using namespace MediaInfoLib;
const size_t Error=(size_t)-1;
//---------------------------------------------------------------------------

//***************************************************************************
// Constructor/Destructor
//***************************************************************************

//---------------------------------------------------------------------------
Core::Core()
{
    MI=new MediaInfoList;
    HTML=true;
}

Core::~Core()
{
    delete MI;
}

//***************************************************************************
// Show Info
//***************************************************************************

//---------------------------------------------------------------------------
void Core::Data_Prepare()
{
    //Inform
    TempText=MI->Inform((size_t)-1).c_str();
    if (TempText.find(_T("<html>"))<4)
        HTML=true;
    else
        HTML=false;

    //Show
    /*
    if (HTML)
    {
        wxFile F;
        if (TempFile==_T(""))
        {
            TempFile=wxFileName::CreateTempFileName(_T("MI_"));
            wxRemoveFile(TempFile.c_str());
            TempFile+=_T(".html");
        }
        F.Open(TempFile.c_str(), wxFile::write);
        F.Write(TempText.c_str(), wxConvUTF8);
        F.Close();
    }
    */
}

//***************************************************************************
// Menu
//***************************************************************************

//---------------------------------------------------------------------------
void Core::Menu_File_Open_File (const MediaInfo_String& FileName)
{
    MI->Close(Error);
    MI->Open(FileName.c_str());
}

//---------------------------------------------------------------------------
void Core::Menu_File_Open_Files_Begin ()
{
    MI->Close(Error);
}

//---------------------------------------------------------------------------
void Core::Menu_File_Open_Files_Continue (const MediaInfo_String& FileName)
{
    MI->Open(FileName.c_str());
}

//---------------------------------------------------------------------------
void Core::Menu_View_HTML ()
{
    MI->Option(_T("Inform"), _T("HTML"));
    HTML=true;
}

//---------------------------------------------------------------------------
void Core::Menu_View_Text ()
{
    MI->Option(_T("Inform"), _T(""));
    HTML=false;
}

//---------------------------------------------------------------------------
void Core::Menu_Option_Preferences_Inform (const MediaInfo_String& Inform)
{
    MI->Option(_T("Inform"), Inform);
}

//---------------------------------------------------------------------------
void Core::Menu_Option_Preferences_Option (const MediaInfo_String& Param, const MediaInfo_String& Value)
{
    MI->Option(Param, Value);
}

//---------------------------------------------------------------------------
void Core::Menu_Debug_Complete (bool Value)
{
    if (Value)
        MI->Option(_T("Complete"), _T("1"));
    else
        MI->Option(_T("Complete"), _T("0"));
}

//---------------------------------------------------------------------------
void Core::Menu_Debug_Details (float Value)
{
    MI->Option(_T("Inform"), MediaInfo_String(_T("Details;0.9"))/*+Ztring::ToZtring(Value)*/);
    HTML=false;
}

//---------------------------------------------------------------------------
MediaInfo_String& Core::Menu_Help_Version ()
{
    TempText=MI->Option(_T("Info_Version"));
    return TempText;
}

//---------------------------------------------------------------------------
MediaInfo_String& Core::Menu_Help_Info_Parameters ()
{
    TempText=MI->Option(_T("Info_Parameters"));
    return TempText;
}

//***************************************************************************
// Helpers
//***************************************************************************

//---------------------------------------------------------------------------
bool Core::HTML_Get ()
{
    return HTML;
}

//---------------------------------------------------------------------------
void Core::HTML_Set (bool HTML_)
{
    HTML=HTML_;
}

//---------------------------------------------------------------------------
MediaInfo_String& Core::TempText_Get ()
{
    return TempText;
}

//---------------------------------------------------------------------------
MediaInfo_String& Core::TempFile_Get ()
{
    return TempFile;
}
